#!/bin/sh

LOG_FILE="/dnake/bin/firmware_flash_log.txt"
OTA_PLACE_HOLDER="/dnake/data/upgrade_status.txt"
REBOOT_PLACE_HOLDER="/dnake/data/reboot_place_holder.txt"

SUCCESS=0
STANDARD_FAILURE=1
CRITICAL_SECTION_FAILURE=2

FILES_MATCH=0
FILES_NO_MATCH=1

upgrade_package_root=""
report_progress_enabled=0
ota_upgrade_enabled=0
reboot_required=0

current_model_not_set=0
copy_ringtones_to_system=0

#
# Logs to file and logcat at Info level.
#
logInfo()
{
    log -p i -t "Upgrade script" $1
    echo $1 >> ${LOG_FILE}
}

#
# Logs to file and logcat at Warning level.
#
logWarning()
{
    log -p w -t "Upgrade script" $1
    echo $1 >> ${LOG_FILE}
}

#
# Logs to file and logcat at Error level.
#
logError()
{
    log -p e -t "Upgrade script" $1
    echo $1 >> ${LOG_FILE}
}

#
# Reports progress to upgrade daemon process via parent pid.
#
reportProgress()
{
    # Send custom signal to upgrade daemon
    if [ ${report_progress_enabled} -eq 1 ]
    then
        kill -USR1 ${PPID}
        if [ $? -ne ${SUCCESS} ]
        then
            logWarning "Failed to report progress"
        fi
    fi
}

#
# Checks if file count in destination directory is valid.
# File count must match or be greater than file count in source directory.
#
check_file_count()
{
    local src_file_count=0
    local dest_file_count=0

    local file_count=0

    count_files_in_dir $1
    src_file_count=${file_count}

    file_count=0

    count_files_in_dir $2
    dest_file_count=${file_count}

    if [ ${dest_file_count} -lt ${src_file_count} ]
    then
        logError "Destination file count ($dest_file_count) less than source file count ($src_file_count)"
        logError "Aborting firmware flashing process!"
        exit 1
    fi
}

#
# Recursively count all files in directory (including sub-directories).
#
count_files_in_dir()
{
    for item in $1/*
    do
        if [ -f "$item" ]
        then
            file_count=$((file_count + 1))
        elif [ -d "$item" ]
        then
            count_files_in_dir ${item}
        fi
    done
}

#
# Compares MD5 hash of source and destination files.
#
compare_files()
{
    if [ -e $2 ]
    then
        # Get md5sum for each file
        src_md5_result="$(busybox md5sum $1 | busybox cut -d ' ' -f1)"
        dest_md5_result="$(busybox md5sum $2 | busybox cut -d ' ' -f1)"

        # Results will contain full path - pass to "cut" to split on spaces and compare first fields
        if [[ ${src_md5_result} != ${dest_md5_result} ]]
        then
            # Files are different
            return ${FILES_NO_MATCH}
        else
            # Files are the same
            return ${FILES_MATCH}
        fi
    else
        # Destination file does not exist
        return ${FILES_NO_MATCH}
    fi
}

#
# Disables system packages using Android's Package Manager.
#
disable_system_package()
{
    # Check if package is enabled - if so, disable
    system_ui_package="$(pm list packages -e $1)"
    # Do wildcard match as pm list packages prepends "package:" to result
    if [[ ${system_ui_package} == *$1* ]]
    then
        # Disable system ui
        logInfo "Disabling package ${system_ui_package}"
        pm disable $1
    fi
}

#
# Upgrades binary files including web interface files.
#
upgrade_binary_files()
{
    logInfo "Installing bin files..."

	# Only copy over build.prop if it has not been previously set up as this file controls model type/product
	if [ ${current_model_not_set} -eq 0 ]
	then
		for entry in `ls ${upgrade_package_root}/dnake/bin`
		do
			if [ ${entry} != "build.prop" ]
			then
				cp -rf ${upgrade_package_root}/dnake/bin/${entry} /dnake/bin
				if [ $? -ne ${SUCCESS} ]
				then
					logError "Failed to copy ${entry} to bin folder"
					return ${STANDARD_FAILURE}
				fi
			fi
		done
	else
		cp -rf ${upgrade_package_root}/dnake/bin/* /dnake/bin/
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to copy bin folder"
			return ${STANDARD_FAILURE}
		fi
	fi

    chmod -R 0777 /dnake/bin/*
    if [ $? -ne ${SUCCESS} ]
    then
        logError "Failed to chmod bin folder"
        return ${STANDARD_FAILURE}
    fi

	if [ -d "${upgrade_package_root}/system/preinstall" ]
	then
		cp -f ${upgrade_package_root}/system/preinstall/* /dnake/bin/
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to copy preinstall folder"
			return ${STANDARD_FAILURE}
		fi
	fi

    chmod 0644 /dnake/bin/*.bak

    logInfo "...done!"
}

#
# Upgrades config files and resets ringtones
#
upgrade_project_files()
{
    logInfo "Installing project files..."

    mkdir -p /dnake/cfg/system/device
    mkdir -p /dnake/cfg/system/preferences
    mkdir -p /dnake/cfg/system/smart_home
    mkdir -p /dnake/cfg/system/ui
    mkdir -p /dnake/data/system
	mkdir -p /dnake/cfg/local/smart_home/schedules
	mkdir -p /dnake/cfg/local/smart_home/scene
	mkdir -p /dnake/cfg/local/smart_home/pairing
	mkdir -p /dnake/cfg/local/smart_home/modules
	mkdir -p /dnake/cfg/local/preferences
	mkdir -p /dnake/data/local/saved

    mount -o remount,rw /system

	# Only need to copy over ringtones if ringtone count has changed
	local src_ringtone_count=0
    local dest_ringtone_count=0
    local file_count=0

    count_files_in_dir /system/media/audio/ringtones/
    src_ringtone_count=${file_count}

    file_count=0

    count_files_in_dir ${upgrade_package_root}/dnake/data/system/ringtones/
    dest_ringtone_count=${file_count}
	
	if [ ${dest_ringtone_count} -ne ${src_ringtone_count} ]
	then
		logInfo "Ringtones have changed - marking for copy"
		copy_ringtones_to_system=1
	fi
	
	cp -rf ${upgrade_package_root}/dnake/data/* /dnake/data

	# If in OTA mode, must make sure the modules.xml file is not overwritten
    if [ ${ota_upgrade_enabled} -eq 1 ]
    then
        logInfo "OTA upgrade - copying config files"

        local modules_file="/dnake/cfg/system/device/modules.xml"

        if [ -e ${modules_file} ]
        then
            for entry in `ls ${upgrade_package_root}/dnake/cfg/system/device`
            do
                if [ ${entry} != ${modules_file} ]
                then
                    cp -f ${upgrade_package_root}/dnake/cfg/system/device/${entry} /dnake/cfg/system/device
                    if [ $? -ne ${SUCCESS} ]
                    then
                        logError "Failed to copy ${entry} to system device folder"
                        return ${STANDARD_FAILURE}
                    fi
                fi
            done
        else
            cp -rf ${upgrade_package_root}/dnake/cfg/system/device/* /dnake/cfg/system/device/
            if [ $? -ne ${SUCCESS} ]
            then
                logError "Failed to copy system device folder"
                return ${STANDARD_FAILURE}
            fi
        fi
		
		cp -rf ${upgrade_package_root}/dnake/cfg/system/preferences/* /dnake/cfg/system/preferences/
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to copy system preferences folder"
			return ${STANDARD_FAILURE}
		fi

		cp -rf ${upgrade_package_root}/dnake/cfg/system/smart_home/* /dnake/cfg/system/smart_home/
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to copy system ui folder"
			return ${STANDARD_FAILURE}
		fi

		cp -rf ${upgrade_package_root}/dnake/cfg/system/ui/* /dnake/cfg/system/ui/
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to copy system ui folder"
			return ${STANDARD_FAILURE}
		fi
		
		cp -f ${upgrade_package_root}/dnake/cfg/*.xml /dnake/cfg/
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to copy cfg folder contents"
			return ${STANDARD_FAILURE}
		fi
	else
        cp -rf ${upgrade_package_root}/dnake/cfg/* /dnake/cfg
        if [ $? -ne ${SUCCESS} ]
        then
            logError "Failed to copy cfg folder"
            return ${STANDARD_FAILURE}
        fi
	fi

    check_file_count ${upgrade_package_root}/dnake/cfg /dnake/cfg

    # Remove redundant sound files
    rm -f /dnake/bin/prompt/alarm_delay.wav
    rm -f /dnake/bin/prompt/defence_cancel.wav
    rm -f /dnake/bin/prompt/defence_delay.wav
    rm -f /dnake/bin/prompt/defence_on.wav
    rm -f /dnake/bin/prompt/modify_failed.wav
    rm -f /dnake/bin/prompt/modify_success.wav
    rm -f /dnake/bin/prompt/passwd_err.wav
	
	
	
	####################################################################
	####################################################################
	######## TEMPORARY CHANGE - REMOVE/EDIT FOR FUTURE VERSIONS ########
	####################################################################
	####################################################################
	
	if [ -e "/dnake/data/project_settings/current_project_settings.xml" ]
	then
		local site_id="$(busybox grep '<name>' dnake/data/project_settings/current_project_settings.xml | busybox awk -F'[<>]' '{print $3}')"
		busybox sed -i "s/<site_id>.*<\/site_id>/<site_id>${site_id}<\/site_id>/" /dnake/cfg/system/device/project.xml
		
		local service="$(busybox grep '<service>' dnake/data/project_settings/current_project_settings.xml | busybox awk -F'[<>]' '{print $3}')"
		
		local modules_file="/dnake/cfg/system/device/modules.xml"
		local line_number=1
		local matched_de_line=0
		local matched_lc_line=0
		while IFS= read -r line
		do
			if [[ "${line}" = *"DE"* ]]
			then
				matched_de_line=($((line_number + 1)))
			elif [[ "${line}" = *"LC"* ]]
			then
				matched_lc_line=($((line_number + 1)))
			fi
			
			((line_number++))
		done < ${modules_file}
		
		if [ "${service}" = "LC" ]
		then
		    logInfo "Upgrading from a previous LC installation"
			busybox sed -i "${matched_de_line}s/0/1/" ${modules_file}
			busybox sed -i "${matched_lc_line}s/0/1/" ${modules_file}
		else
		    logInfo "Upgrading from a previous DE installation"
			busybox sed -i "${matched_de_line}s/0/1/" ${modules_file}
		fi
		
		rm -rf /dnake/data/project_settings
	
		if [ -e "/dnake/data/sys.xml" ]
		then
			rm -f /dnake/data/sys.xml
		fi

		if [ -e "/dnake/cfg/network_setup.xml" ]
		then
			cp -f /dnake/cfg/network_setup.xml /dnake/cfg/system/device
			chmod 777 /dnake/cfg/system/device/network_setup.xml
			rm -f /dnake/cfg/network_setup.xml
		fi

		if [ -e "/dnake/cfg/ntp.xml" ]
		then
			cp -f /dnake/cfg/ntp.xml /dnake/cfg/system/device
			chmod 777 /dnake/cfg/system/device/ntp.xml
			rm -f /dnake/cfg/ntp.xml
		else
            local ntp_server=$(settings get global ntp_server)
            if [ ${ntp_server} != "null" ] && [ ${ntp_server} != "NULL" ] && [ ${ntp_server} != "" ]
            then
                echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?><ntp><address>${ntp_server}</address></ntp>" > /dnake/cfg/system/device/ntp.xml
            fi
        fi

		if [ -e "/dnake/cfg/location.xml" ]
		then
			cp -f /dnake/cfg/location.xml /dnake/cfg/system/device
			chmod 777 /dnake/cfg/system/device/location.xml
			rm -f /dnake/cfg/location.xml
		fi

        if [ -e "/dnake/cfg/setup.xml" ]
        then
            cp -f /dnake/cfg/setup.xml /dnake/cfg/system/device
            chmod 777 /dnake/cfg/system/device/setup.xml
            rm -f /dnake/cfg/setup.xml
        fi

        if [ -e "/dnake/cfg/smart.xml" ]
        then
            cp -f /dnake/cfg/smart.xml /dnake/cfg/local/smart_home/modules/room.xml
            rm -f /dnake/cfg/smart.xml
        fi

        if [ -e "/dnake/cfg/slave_device.xml" ]
        then
            cp -f /dnake/cfg/slave_device.xml /dnake/cfg/local/smart_home/pairing/devices.xml
            rm -f /dnake/cfg/slave_device.xml
        fi

        if [ -d "/dnake/data/scene_data" ]
        then
            if [ -e "/dnake/data/scene_data/scene_cfg.xml" ]
            then
                cp -f /dnake/data/scene_data/scene_cfg.xml /dnake/cfg/local/smart_home/scene
                busybox sed -i "s/<iconName>setup_icon_light<\/iconName>/<iconName>scene_icon_light_off<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>light_off<\/iconName>/<iconName>scene_icon_light_off<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>light_low<\/iconName>/<iconName>scene_icon_light_low<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>light_on<\/iconName>/<iconName>scene_icon_light_on<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>night<\/iconName>/<iconName>scene_icon_night<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>bathrobe<\/iconName>/<iconName>scene_icon_bathrobe<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>morning<\/iconName>/<iconName>scene_icon_morning<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>popcorn<\/iconName>/<iconName>scene_icon_popcorn<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>disco_ball<\/iconName>/<iconName>scene_icon_party<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
                busybox sed -i "s/<iconName>cutlery<\/iconName>/<iconName>scene_icon_dinner<\/iconName>/g" /dnake/cfg/local/smart_home/scene/scene_cfg.xml
            fi

            if [ -e "/dnake/data/scene_data/scene_data.xml" ]
            then
                cp -f /dnake/data/scene_data/scene_data.xml /dnake/cfg/local/smart_home/scene/old_scene_data.xml
            fi

            rm -rf /dnake/data/scene_data
        fi

        if [ -d "/dnake/data/user_settings" ]
        then
            if [ -e "/dnake/data/user_settings/user_settings.xml" ]
            then
                cp -f /dnake/data/user_settings/user_settings.xml /dnake/cfg/local/preferences
                chmod 777 /dnake/cfg/local/preferences/user_settings.xml
            fi

            rm -rf /dnake/data/user_settings
        fi

        if [ -d "/dnake/data/avi" ]
        then
            if [ -e "/dnake/data/avi/logger.xml" ]
            then
                cp -f /dnake/data/avi/logger.xml /dnake/data/local/saved/old_recordings.xml
                chmod 777 /dnake/data/local/saved/old_recordings.xml
            fi

            rm -rf /dnake/data/avi
        fi

        if [ -d "/dnake/data/jpeg" ]
        then
            if [ -e "/dnake/data/jpeg/logger.xml" ]
            then
                cp -f /dnake/data/jpeg/logger.xml /dnake/data/local/saved/old_screenshots.xml
                chmod 777 /dnake/data/local/saved/old_screenshots.xml
            fi

            rm -rf /dnake/data/jpeg
        fi

        if [ -d "/dnake/data/notifications" ]
        then
            if [ -e "/dnake/data/notifications/logger.xml" ]
            then
                cp -f /dnake/data/notifications/logger.xml /dnake/data/local/saved/old_notifications.xml
                chmod 777 /dnake/data/local/saved/old_notifications.xml
            fi

            rm -rf /dnake/data/notifications
        fi

        if [ -d "/dnake/data/images" ]
        then
            cp -rf /dnake/data/images/ /dnake/data/system/ui/images
            rm -rf /dnake/data/images
        fi

        rm -f /dnake/cfg/apps.xml
        rm -f /dnake/cfg/*.crt
        rm -rf /dnake/cfg/lost+found
        rm -rf /dnake/cfg/versions
        rm -rf /dnake/data/lost+found
        rm -rf /dnake/data/talk
        rm -rf /dnake/data/text
	fi


	####################################################################
	####################################################################
	######## TEMPORARY CHANGE - REMOVE/EDIT FOR FUTURE VERSIONS ########
	####################################################################
	####################################################################

    logInfo "...done!"
}

#
# Upgrades system settings including setting correct time zones, NTP, default ringtone and screen brightness.
#
upgrade_system_settings()
{
    logInfo "Configuring system settings..."

    # Turn off screen sleep
    settings put system screen_off_timeout 2147483000

    # Force to Europe/London
    setprop persist.sys.timezone "Europe/London"
    setprop persist.sys.locale en-GB
    setprop persist.sys.language en
    setprop persist.sys.country GB

    # Set ADB to launch in tcp mode for remote debugging
    setprop persist.adb.tcp.port 5555

    if [ ${ota_upgrade_enabled} -eq 0 ]
    then
        logInfo "Standard upgrade - force reset of screen brightness and date/time"

        # Set default screen brightness
        settings put system screen_brightness 175

        # Ensure auto time is on for NTP provided time, disable auto time zone
        settings put global auto_time 1
        settings put global time_zone 0

        # Set date to default value (should be overridden by NTP)
        date -s 20170101.120000
    fi

    # Populate ntp server with default if not already set
    local ntp_server=$(settings get global ntp_server)
    if [ ${ntp_server} == "null" ] || [ ${ntp_server} == "NULL" ] || [ ${ntp_server} == "" ]
    then
        settings put global ntp_server 2.android.pool.ntp.org
    fi

    logInfo "...done!"
}

#
# Upgrades critical system files.
# Since there is a risk of unexpected system crashes when modifying these files, files are only modified if needed.
#
upgrade_system_files()
{
    #------------------------------------------------------------
    #mount -o remount,rw /system

    #cp -f ${upgrade_package_root}/system/vendor/modules/* /system/vendor/modules/
    #chmod 0644 /system/vendor/modules/*.ko

    logInfo "Installing critical system files..."

    rm -f /system/app/toolbar.apk
    rm -f /system/app/apps.apk
    rm -f /system/app/d400.apk
    rm -f /data/app/*.apk

    # Start copying over system files ONLY if they are different to incoming source files
    for item in ${upgrade_package_root}/system/app/*
    do
        # Strip out path name before passing file name as 2nd parameter to compare_files()
        file="$(busybox basename "$item")"
        compare_files ${item} /system/app/${file}
        if [ $? -ne ${FILES_MATCH} ]
        then
            logInfo "${file} source and destination files do not match - copying over"
            rm -f /system/app/${file}
            cp -f ${item} /system/app/${file}
            if [ $? -ne ${SUCCESS} ]
            then
                logError "Failed to copy ${file} to system folder"
                return ${CRITICAL_SECTION_FAILURE}
            fi
        fi
    done

    chmod 0644 /system/app/*.apk

    if [ ${ota_upgrade_enabled} -eq 0 ]
    then
        logInfo "Standard upgrade - checking bootanimation files"

        #compare_files ${upgrade_package_root}/system/etc/bootanimation/bootanimation /system/bin/bootanimation
        #if [ $? -ne ${FILES_MATCH} ]
        #then
        #    logInfo "Bootanimation source and destination file sizes do not match - copying over"
        #    cp -f ${upgrade_package_root}/system/etc/bootanimation/bootanimation /system/bin/bootanimation
        #    if [ $? -ne ${SUCCESS} ]
        #    then
        #        logError "Failed to copy bootanimation"
        #        return ${CRITICAL_SECTION_FAILURE}
        #    fi
        #fi

        #chmod 0755 /system/bin/bootanimation
        #if [ $? -ne ${SUCCESS} ]
        #then
        #    logError "Failed to chmod bootanimation"
        #    return ${STANDARD_FAILURE}
        #fi

        compare_files ${upgrade_package_root}/system/etc/bootanimation/bootanimation.zip /system/media/bootanimation.zip
        if [ $? -ne ${FILES_MATCH} ]
        then
            logInfo "Bootanimation.zip source and destination file sizes do not match - copying over"
            cp -f ${upgrade_package_root}/system/etc/bootanimation/bootanimation.zip /system/media/bootanimation.zip
            if [ $? -ne ${SUCCESS} ]
            then
                logError "Failed to copy bootanimation zip"
                return ${CRITICAL_SECTION_FAILURE}
            fi
        fi

        chmod 0644 /system/media/bootanimation.zip
        if [ $? -ne ${SUCCESS} ]
        then
            logError "Failed to chmod bootanimation zip"
            return ${STANDARD_FAILURE}
        fi
    fi

    for item in ${upgrade_package_root}/system/lib/*
    do
        # Strip out path name before passing file name as 2nd parameter to compare_files()
        file="$(busybox basename "$item")"
        compare_files ${item} /system/lib/${file}
        if [ $? -ne ${FILES_MATCH} ]
        then
            logInfo "${file} source and destination file sizes do not match - copying over"
            cp -f ${item} /system/lib/${file}
            if [ $? -ne ${SUCCESS} ]
            then
                logError "Failed to copy ${file} to libs folder"
                return ${CRITICAL_SECTION_FAILURE}
            fi
        fi
    done

    chmod 0644 /system/lib/*.so
	
    # Disable system ui
    disable_system_package "com.android.systemui"
	
    # Disable pinyin input
    disable_system_package "com.google.android.inputmethod.pinyin"

    if [ ${copy_ringtones_to_system} -eq 1 ]
    then
        logInfo "Copying over new ringtones"
		# Remove existing ringtones and replace with custom ones
		rm -f /system/media/audio/ringtones/*
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to delete old ringtones"
			return ${STANDARD_FAILURE}
		fi

		cp -rf ${upgrade_package_root}/dnake/data/system/ringtones/* /system/media/audio/ringtones/
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to copy ringtones"
			return ${STANDARD_FAILURE}
		fi

		chmod 0644 /system/media/audio/ringtones/*
		if [ $? -ne ${SUCCESS} ]
		then
			logError "Failed to chmod ringtones"
			return ${STANDARD_FAILURE}
		fi

		check_file_count ${upgrade_package_root}/dnake/data/system/ringtones /system/media/audio/ringtones

        # Clear default ringtone - default ringtone automatically set on boot
        settings put system ringtone ""

		# Delete internal db - will force system to create new db which will include references to custom ringtones
		rm -f data/data/com.android.providers.media/databases/internal*
    fi

    logInfo "...done!"
}

#
# Starts upgrade process.
# Currently composes of 4 main stages.
#
start_upgrade_process()
{
    logInfo "Firmware flashing process started!"

    upgrade_system_settings
    if [ $? -ne ${SUCCESS} ]
    then
        return ${STANDARD_FAILURE}
    fi

    reportProgress

    upgrade_project_files
    if [ $? -ne ${SUCCESS} ]
    then
        return ${STANDARD_FAILURE}
    fi

    reportProgress

    upgrade_binary_files
    if [ $? -ne ${SUCCESS} ]
    then
        return ${STANDARD_FAILURE}
    fi

    reportProgress

    upgrade_system_files
    if [ $? -ne ${SUCCESS} ]
    then
        return ${CRITICAL_SECTION_FAILURE}
    fi

    reportProgress

    logInfo "Firmware flashing process succeeded!"
}

#
# Checks argument validity
#
check_arguments()
{
    if [[ $# -eq 0 ]]
    then
        logError "No command arguments provided"
        exit 1
    fi
	
	while [[ $# -gt 0 ]]
	do
		key="$1"
		echo $1

		case ${key} in
			-i|--install)
                upgrade_package_root=$2
                logInfo "Upgrade package = ${upgrade_package_root}"
				shift # past argument
				shift # past value
				;;
            -r | --report)
                report_progress_enabled=1
                logInfo "Reporting enabled"
				shift # past argument
				shift # past value
				;;
            -ota)
                ota_upgrade_enabled=1
                logInfo "OTA upgrade enabled"
				shift # past argument
				shift # past value
				;;
			*)
				# Check for legacy command first
                if [[ $1 = *"udata"* ]]
                then
                    upgrade_package_root=$1
                    logInfo "Upgrade package = ${upgrade_package_root}"
					break
                else
                    logError "Unknown command received - ($1)"
                    exit 1
                fi
                ;;
		esac
	done
}

#
# Checks validity of incoming firmware file
#
check_firmware_validity()
{
	local current_model_search=$(grep "ro.product.model" /system/build.prop)
	local current_model=${current_model_search#*=}

	local incoming_model_search=$(grep "ro.product.model" ${upgrade_package_root}/dnake/bin/build.prop)
	local incoming_model=${incoming_model_search#*=}
	
	logInfo "Firmware validaty check - current model = ${current_model}, incoming model = ${incoming_model}"
	if [ ${current_model} != "ORC-4T" ] && [ ${current_model} != "ORC-7T" ] && [ ${current_model} != "ORC-7TF" ] && [ ${current_model} != "ORC-10T" ] \
		&& [ ${current_model} != "ORC-E2v2" ] && [ ${current_model} != "ORC-E3v2" ] && [ ${current_model} != "ORC-E5" ]
	then
		logInfo "Current model not set - assuming firmware is correct for model. Proceeding with flashing process"
		current_model_not_set=1
	elif [ ${current_model} != ${incoming_model} ]
	then
		logError "Current model does not match incoming model, incorrect firmware detected! Aborting flashing process!"
		exit 1
	else
		logInfo "Current model matches incoming model - proceeding with flashing process"
	fi
}

#
# Checks current firmware version and sets flags accordingly
#
check_firmware_version()
{
    if [ -d "/dnake/cfg/versions" ] && [ ${upgrade_package_root} == "/var/upgrade/temp/udata" ]
    then
        logInfo "Detected legacy firmware for OTA upgrade - setting OTA flag"
        local smart_version_minor2="$(busybox grep '<version_minor2>' dnake/cfg/versions/smart/version.xml | busybox awk -F'[<>]' '{print $3}')"
        ota_upgrade_enabled=1
        if [ ${smart_version_minor2} -eq 1 ]
        then
            logInfo "Legacy firmware requires manual reboot - setting reboot flag"
            reboot_required=1
        fi
    fi
}

mount -o remount,rw /dnake/bin

rm -f ${LOG_FILE}
rm -f ${REBOOT_PLACE_HOLDER}

touch ${LOG_FILE}
timestamp=$(date -u +"%d-%m-%y:%T")
logInfo "Firmware flashing process starting at ${timestamp}"

check_arguments $@
check_firmware_version
check_firmware_validity
start_upgrade_process

# Store result of upgrade process and check
RESULT=$?
if [ ${RESULT} -eq ${STANDARD_FAILURE} ]
then
    # Standard failure occurred - try one more time
    logInfo "Retrying flashing process"
    start_upgrade_process
    if [ $? -ne ${SUCCESS} ]
    then
        # Failed again - abort
        logError "Aborting firmware flashing process!"
        exit 1
    else
        if [ ${ota_upgrade_enabled} -eq 1 ]
        then
            echo 'true' > ${OTA_PLACE_HOLDER}
        fi

        echo 'true' > ${REBOOT_PLACE_HOLDER}
    fi
elif [ ${RESULT} -eq ${CRITICAL_SECTION_FAILURE} ]
then
    # Failure occurred during "critical" section - abort
    logError "Critical failure - Aborting firmware flashing process!"
    exit 1
else
    if [ ${ota_upgrade_enabled} -eq 1 ]
    then
        echo 'true' > ${OTA_PLACE_HOLDER}
    fi

    echo 'true' > ${REBOOT_PLACE_HOLDER}
fi

if [ ${reboot_required} -eq 1 ]
then
    reboot
fi
